from fastapi import APIRouter, HTTPException, Depends
from sqlmodel import Session
from app.backend.schemas.user import UserResponse, UserUpdate
from backend.database import get_session
from backend.models.models import User
from backend.routes.auth import get_current_user, admin_required

router = APIRouter()


# Get all users
@router.get("/", response_model=list[UserResponse])
def get_all_users(db: Session = Depends(get_session)):
    users = db.query(User).all()
    return users


# Get one user by ID
@router.get("/{user_id}", response_model=UserResponse)
def get_user(user_id: int, db: Session = Depends(get_session)):
    user = db.query(User).filter(User.id == user_id).first()
    if not user:
        raise HTTPException(status_code=404, detail="User not found")
    return user


# Update a user
@router.put("/{user_id}", response_model=UserResponse)
def update_user(
    user_id: int, user_update: UserUpdate, db: Session = Depends(get_session)
):
    user = db.query(User).filter(User.id == user_id).first()
    if not user:
        raise HTTPException(status_code=404, detail="User not found")
    for key, value in user_update.dict(exclude_unset=True).items():
        setattr(user, key, value)
    db.commit()
    db.refresh(user)
    return user


# Update a user's role (Admin only)
@router.patch("/{user_id}/role", response_model=UserResponse)
def update_user_role(
    user_id: int,
    new_role: str,
    db: Session = Depends(get_session),
    current_user: User = Depends(admin_required),  # Ensure only admins can access
):
    # Validate the new role
    valid_roles = ["customer", "shop_owner", "admin"]
    if new_role not in valid_roles:
        raise HTTPException(status_code=400, detail="Invalid role")

    # Fetch the user
    user = db.query(User).filter(User.id == user_id).first()
    if not user:
        raise HTTPException(status_code=404, detail="User not found")

    # Update the role
    user.role = new_role
    db.add(user)
    db.commit()
    db.refresh(user)
    return user


# Delete a user
@router.delete("/{user_id}", status_code=204)
def delete_user(user_id: int, db: Session = Depends(get_session)):
    user = db.query(User).filter(User.id == user_id).first()
    if not user:
        raise HTTPException(status_code=404, detail="User not found")
    db.delete(user)
    db.commit()
    return
